# Global import
from typing import Union

# Local import
from src.shape_calculator.arithops import ArithOps


class ArithOpsExtended(ArithOps):
    """Overrides ArithOps and adds three arithmetic operations: exponentiation, modulus, and floor division."""

    def __check(self, *argv: Union[float, int]):
        """Explicitly checking the validity of the Arithmetic Methods.

        Mainly for testing purposes. Is not run upon instantiation.

        Raises an Error if the value of the fields are not valid.

        Args:
            value (float|int): Value to use one of the ArithOps
        Raises:
            ValueError: If the value to divide is zero or the value to use is zero, the calculation is invalid.
        """
        for arg in argv:
            if arg == 0:
                raise ValueError("Cannot use zero.")

    def add(self, *argv: Union[float, int]):
        """Add the value to instance value called `value`.

        Args:
            value (float|int): Value to add
        """
        for arg in argv:
            self.value += arg

    def sub(self, *argv: Union[float, int]):
        """Subtract the value to the instance value called `value`.

        Args:
            value (float|int): Value to subtract
        """
        for arg in argv:
            self.value -= arg

    def mul(self, *argv: Union[float, int]):
        """Multiply the value to instance value called `value`.

        Args:
            value (float|int): Value to multiply
        """
        for arg in argv:
            self.value *= arg

    def div(self, *argv: Union[float, int]):
        """Divide the value to the instance value called `value`.

        Args:
            value (float|int): Value to divide

        Raises:
            ZeroDivisionError: If the value to divide is zero, the calculation is invalid.
        """
        self.__check(argv)
        for arg in argv:
            self.value /= arg

    def exp(self, *argv: Union[float, int]):
        """Exponentiate the value to instance value called `value`.

        Args:
            value (float|int): Value to exponentiate
        """
        self.__check(self.value)
        for arg in argv:
            self.value = self.value**arg

    def mod(self, *argv: Union[float, int]):
        """Get the remainder of dividing the value to instance value called `value`.

        Args:
            value (float|int): Value to modulus
        """
        self.__check(argv)
        for arg in argv:
            self.value = self.value % arg

    def floordiv(self, *argv: Union[float, int]):
        """Get the floor or the lower number for the value to instance value called `value`.

        Args:
            value (float|int): Value to floor division
        """
        self.__check(argv)
        for arg in argv:
            self.value = self.value // arg



# Global import
from typing import Union

from src.shape_calculator.arithops_extended import ArithOpsExtended

# Local import
from src.shape_calculator.shapes.shape import Shape


class Triangle(Shape):
    """Represents a triangle, calculating area and perimeter from the abstract class Shape."""

    def __init__(
        self,
        base: Union[float, int],
        height: Union[float, int],
        side1: Union[float, int],
        side2: Union[float, int],
        side3: Union[float, int],
    ):
        """Initializes a Triangle object.

        Args:
        - base (float|int): the length of the base of the triangle
        - height (float|int): the height of the triangle
        - side1 (float|int): the length of the first side of the triangle
        - side2 (float|int): the length of the second side of the triangle
        - side3 (float|int): the length of the third side of the triangle
        """
        self.base = base
        self.height = height
        self.side1 = side1
        self.side2 = side2
        self.side3 = side3
        self.__check()
        self.arithops = ArithOpsExtended()

    def __check(self):
        """Checks if the input values are valid for a triangle.

        Raises:
            ValueError: If the input values are not positive.
            ValueError: If the input values are invalid for a triangle.
        """
        if (
            self.base <= 0
            or self.height <= 0
            or self.side1 <= 0
            or self.side2 <= 0
            or self.side3 <= 0
        ):
            raise ValueError("Input values should be positive.")
        if (
            self.side1 + self.side2 <= self.base
            or self.side1 + self.side3 <= self.base
            or self.side2 + self.side3 <= self.base
        ):
            raise ValueError("Invalid input values for triangle.")

    def get_area(self) -> Union[float, int]:
        """Calculates the area of the triangle.

        Returns:
            Union[float, int]: The area of the triangle.
        """
        self.arithops.value = self.base
        self.arithops.mul(self.height)
        self.arithops.div(2)
        return self.arithops.value

    def get_perimeter(self) -> Union[float, int]:
        """Calculates the perimeter of the triangle.

        Returns:
            Union[float, int]: The perimeter of the triangle.
        """
        self.arithops.value = 0
        self.arithops.add(self.side1, self.side2, self.side3)
        return self.arithops.value



import click

from src.shape_calculator.shapes.rectangle import Rectangle
from src.shape_calculator.shapes.triangle import Triangle


@click.command()
@click.option("--count", default=1, prompt="Frequency", help="Number of frequency to calculate.")
@click.option(
    "--shape",
    default="y",
    type=click.Choice(["y", "n"]),
    prompt='Rectangle? Triangle if "n"',
    help="The shape to choose.",
)
@click.option(
    "--value1",
    default=3,
    prompt="width or base",
    help="If the shape is rectangle, the value should be width. If the shape is triangle, the value should be base.",
)
@click.option(
    "--value2",
    default=4,
    prompt="height or height",
    help="If the shape is rectangle, the value should be height. If the shape is triangle, the value should be height.",
)
@click.option(
    "--side1", default=1, prompt="side1", help="If the shape is triangle, the value should be side1."
)
@click.option(
    "--side2", default=2, prompt="side2", help="If the shape is triangle, the value should be side2."
)
@click.option(
    "--side3", default=3, prompt="side3", help="If the shape is triangle, the value should be side3."
)
def cli_shape(count, shape, value1, value2, side1, side2, side3):
    """Simple cli program that calculates from Shape class for a total of COUNT times."""
    for _ in range(count):
        if shape == "y":
            click.echo("Rectangle Calculated, width: {}, height: {}".format(value1, value2))
            rectangle = Rectangle(value1, value2)
            click.echo(rectangle.__repr__())
        else:
            click.echo(
                "Triangle Calculated, base: {}, height: {}, side1: {}, side2: {}, side3: {}".format(
                    value1, value2, side1, side2, side3
                )
            )
            triangle = Triangle(value1, value2, side1, side2, side3)
            click.echo(triangle.__repr__())



from typing import Union


class ArithOps:
    """A class that provides four arithmetic operations: addition, subtraction, multiplication, and division."""

    def __init__(self, value: Union[float, int] = 0):
        """Initialize the instance value called `value` for calculating the arithmetic operations.

        Args:
            value (Union[float, int]): Initial value. Defaults to 0.
        """
        self.value = value

    def add(self, value: Union[float, int]):
        """Add the value to instance value called `value`.

        Args:
            value (Union[float, int]): Value to add
        """
        self.value = self.value + value

    def sub(self, value: Union[float, int]):
        """Subtract the value to the instance value called `value`.

        Args:
            value (Union[float, int]): Value to subtract
        """
        self.value = self.value - value

    def mul(self, value: Union[float, int]):
        """Multiply the value to instance value called `value`.

        Args:
            value (Union[float, int]): Value to multiply
        """
        self.value = self.value * value

    def div(self, value: Union[float, int]):
        """Divide the value to the instance value called `value`.

        Args:
            value (Union[float, int]): Value to divide

        Raises:
            ZeroDivisionError: If the value to divide is zero, the calculation is invalid.
        """
        if value == 0:
            raise ZeroDivisionError("Cannot divide by zero.")
        self.value = self.value / value



from abc import ABC, abstractmethod


class Shape(ABC):
    """Abstract class that represents a shape."""

    @abstractmethod
    def get_area(self):
        """Abstract method that calculates the area of a shape."""
        pass

    @abstractmethod
    def get_perimeter(self):
        """Abstract method that calculates the perimeter of a shape."""
        pass

    def __str__(self):
        """Returns a string representation of the shape."""
        return f"Area: {self.get_area()}\nPerimeter: {self.get_perimeter()}"

    def __repr__(self):
        """Returns a string representation of the shape."""
        return self.__str__()

    # def helper(self):
    #     """Helper method that prints the area and perimeter of the shape."""
    #     print(self.__str__())



# Global import
from typing import Union

from src.shape_calculator.arithops_extended import ArithOpsExtended

# Local import
from src.shape_calculator.shapes.shape import Shape


class Rectangle(Shape):
    """Represents a rectangle, calculating area and perimeter from the abstract class Shape."""

    def __init__(self, length: Union[float, int], width: Union[float, int]):
        """Initializes a Rectangle object.

        Args:
        - length (float|int): the length of the rectangle
        - width (float|int): the width of the rectangle
        """
        self.length = length
        self.width = width
        self.__check()
        self.arithops = ArithOpsExtended()

    def __check(self):
        """Checks if the input values are valid for a triangle.

        Raises:
            ValueError: If the input values are not positive.
        """
        if self.length <= 0 or self.width <= 0:
            raise ValueError("Input values should be positive.")

    def get_area(self) -> Union[float, int]:
        """Calculates the area of the rectangle.

        Returns:
            Union[float, int]: The area of the rectangle.
        """
        self.arithops.value = self.length
        self.arithops.mul(self.width)
        return self.arithops.value

    def get_perimeter(self) -> Union[float, int]:
        """Calculates the perimeter of the rectangle.

        Returns:
            Union[float, int]: The perimeter of the rectangle.
        """
        self.arithops.value = self.length
        self.arithops.add(self.width)
        self.arithops.mul(2)
        return self.arithops.value
